﻿using System;

namespace SI_Class
{
	/// <summary> Simple formatting and parsing of natural numbers using some of the
	/// International System of Units suffixes common in computer use.
	/// Approximates and abbreviates positive integer numbers.
	/// Limits ToString to 4 characters or less with 2 or 3 digits precision.
	/// Parse positive integer numbers from abbreviated string.
	/// A suffix uses powers of 1000 not 2^10, is always capitalized, and is not separated by a space.
	/// </summary>
	static class SI
	{
		/// <summary> Format positive integer numbers to 4 characters or less using SI suffix, if needed.
		/// Automatic override type cast.
		/// </summary>
		/// <param name="number">The numeric value to convert to string</param> 
		public static String ToString (double number)
		{
			if (0 > number)
				return "";
			return ToString ((UInt64) number);
		}
		/// <summary> Format positive integer numbers to 4 characters or less using SI suffix, if needed.
		/// Automatic override type cast.
		/// </summary>
		/// <param name="number">The numeric value to convert to string</param> 
		public static String ToString (Int64 number)
		{
			if (0 > number)
				return "";
			return ToString ((UInt64) number);
		}
		/// <summary> Format positive integer numbers to 4 characters or less using SI suffix, if needed.
		/// </summary>
		/// <param name="number">The numeric value to convert to string</param> 
		public static String ToString (UInt64 number)
		{
			if (10000 > number)
				return number.ToString ();
#if false	// integer only so use alternate floating point
			if (500 < number % 1000)
				number = 1 + number / 1000;
			else
				number = number / 1000;
			if (1000 > number)
				return number.ToString () + "K";
			number = (number + 500) / 1000;
			if (1000 > number)
				return number.ToString () + "M";
			number = (number + 500) / 1000;
			if (1000 > number)
				return number.ToString () + "G";
			number = (number + 500) / 1000;
			if (1000 > number)
				return number.ToString () + "T";
			number = (number + 500) / 1000;
			if (1000 > number)
				return number.ToString () + "P";
			number = (number + 500) / 1000;
			return number.ToString () + "E";
#else	// sometimes provides an extra digit of significance
			double temp = number / 1000.0;
			if (1000 > temp)
				return SingleDecimal (temp) + "K";
			temp = temp / 1000;
			if (1000 > temp)
				return SingleDecimal (temp) + "M";
			temp = temp / 1000;
			if (1000 > temp)
				return SingleDecimal (temp) + "G";
			temp = temp / 1000;
			if (1000 > temp)
				return SingleDecimal (temp) + "T";
			temp = temp / 1000;
			if (1000 > temp)
				return SingleDecimal (temp) + "P";
			temp = temp / 1000;
			return SingleDecimal (temp) + "E";
#endif
		}
		/// <summary> Provide a single decimal place for single digit numbers
		/// </summary>
		/// <param name="number"></param>
		/// <returns></returns>
		static string SingleDecimal (double number)
		{
			if (10 <= number)
				return number.ToString ("F0");
			return number.ToString ("F1");
		}
		/// <summary> Parse the value of an SI string 
		/// </summary>
		/// <param name="str">The string value to convert to numeric</param> 
		public static Int32 ParseI32 (String str)
		{
			UInt64 number = ParseU64 (str);
			if (Int32.MaxValue < number)
				return Int32.MaxValue;
			return (Int32) number;
		}
		/// <summary> Parse the value of an SI string 
		/// </summary>
		/// <param name="str">The string value to convert to numeric</param> 
		public static UInt32 ParseU32 (String str)
		{
			UInt64 number = ParseU64 (str);
			if (UInt32.MaxValue < number)
				return UInt32.MaxValue;
			return (UInt32) number;
		}
		/// <summary> Parse the value of an SI string 
		/// </summary>
		/// <param name="str">The string value to convert to numeric</param> 
		public static Int64 ParseI64 (String str)
		{
			UInt64 number = ParseU64 (str);
			if (Int64.MaxValue < number)
				return Int64.MaxValue;
			return (Int64) number;
		}
		/// <summary> Parse the value of an SI string.
		/// Multiple digits of precision allowed.
		/// </summary>
		/// <param name="str">The string value to convert to numeric</param> 
		public static UInt64 ParseU64 (String str)
		{
			if (null == str)
				return 0;
			int i = str.Length - 1;
			if (i < 0)
				return 0;
			char suffix = str[i];
			UInt64 number = 0;
			if (char.IsDigit (suffix))
			{
				UInt64.TryParse (str, out number);
				return number;
			}
#if false	// integer only so use alternate floating point
			if (!UInt64.TryParse (str.Substring (0, i), out t))
				return 0;
			t *= 1000;
			if ('K' == suffix)
				return t;
			t *= 1000;
			if ('M' == suffix)
				return t;
			t *= 1000;
			if ('G' == suffix)
				return t;
			t *= 1000;
			if ('T' == suffix)
				return t;
			t *= 1000;
			if ('P' == suffix)
				return t;
			t *= 1000;
			if ('E' == suffix)
				return t;
#else	// will process many more digits of precision than ToString creates
			double temp;
			if (!double.TryParse (str.Substring (0, i), out temp))
				return 0;
			if (temp < 0)
				return 0;
			number = (UInt64) (temp * 1000);
			if ('K' == suffix)
				return number;
			number *= 1000;
			if ('M' == suffix)
				return number;
			number *= 1000;
			if ('G' == suffix)
				return number;
			number *= 1000;
			if ('T' == suffix)
				return number;
			number *= 1000;
			if ('P' == suffix)
				return number;
			number *= 1000;
			if ('E' == suffix)
				return number;
#endif
			return 0;
		}
	}
}
